﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace TechNote71_DataConvert_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // 일반 데이터 변수 선언
        private int[] m_iColPos = new int[4];
        private int m_iValue1 = 0;          // int형 데이터 (4Byte)
        private float m_fValue1 = 0;        // float형 데이터 (4Byte)
        private string m_strValue1 = "";    // string(Unicode)형 데이터(6자리 최대12Byte)
        private byte[] m_bySendData1;       // 송신 데이터 저장 바이트배열(구조체X)
        private byte[] m_bySendData2;       // 송신 데이터 저장 바이트배열(구조체O)
        private int m_iSendCount;           // 반복 송신 카운트

        // 구조체 멤버변수에 값을 전달할 임시 변수
        private int m_iValueTemp = 0;
        private float m_fValueTemp = 0;
        private string m_strValueTemp = "";

        // 구조체 ByteArray 선언
        private struct StructureData
        {
            public int m_iValue2;           // int형 데이터 (4Byte)
            public float m_fValue2;         // float형 데이터 (4Byte)
            public byte m_byLength;         // m_strValue2의 사이즈 (1Byte)
            public string m_strValue2;      // string(Unicode)형 데이터(최대6자리. 12Byte)
        }

        // 송신측 구조체선언
        StructureData sSendStruct;
        // 수신측 구조체선언
        StructureData sRecvStruct;

        // Port Number
        private string[] m_strPortNo = { "COM1", "COM2", "COM3", "COM4", "COM7" };
        // BaudRate
        private string[] m_strBaudRate = { "110", "300", "600", "1200", "2400", "4800", "9600", "14400", "19200", "38400", "56000", "57600", "115200" };

        private void Form1_Load(object sender, EventArgs e)
        {
            // 현재 Form을 MainForm으로 설정합니다.
            smartForm1.MainForm = this;

            // ComboBox1에 포트번호 표시
            cmbPortNo.DataSource = m_strPortNo;
            cmbPortNo.SelectItemIndex = 2;

            // ComboBox2에 통신포트 속도 초기값 표시
            cmbBaudRate.DataSource = m_strBaudRate;
            cmbBaudRate.SelectItemIndex = 6;

            // SmartListBox Column스타일 지정
            SmartX.SmartListBox.COLUMNSTYLE[] sColumnStyle = new SmartX.SmartListBox.COLUMNSTYLE[2];

            sColumnStyle[0].eColumnAlign = SmartX.SmartListBox.COLUMNALIGNS.CENTER;
            sColumnStyle[0].iColumnWidth = 296;
            sColumnStyle[0].iOffsetColumn = 0;

            sColumnStyle[1].eColumnAlign = SmartX.SmartListBox.COLUMNALIGNS.CENTER;
            sColumnStyle[1].iColumnWidth = 419;
            sColumnStyle[1].iOffsetColumn = 0;

            smartListBox1.ColumnStyles = sColumnStyle;

            // 일반 데이터 생성(송신용)
            // int형 데이터 (4Byte)
            m_iValue1 = Convert.ToInt32(labInt1.Text);
            // float형 데이터 (4Byte)
            m_fValue1 = Convert.ToSingle(labFloat1.Text);
            // string(Unicode)형 데이터 (8Byte)
            m_strValue1 = labUnicode1.Text;
            MakeNormalData(m_iValue1, m_fValue1, m_strValue1);

            // 구조체 데이터 생성(송신용)
            // int형 데이터 (4Byte)
            m_iValueTemp = Convert.ToInt32(labInt2.Text);
            // float형 데이터 (4Byte)
            m_fValueTemp = Convert.ToSingle(labFloat2.Text);
            // string(Unicode)형 데이터 (최대12Byte)
            m_strValueTemp = labUnicode2.Text;
            MakeStructureData(m_iValueTemp, m_fValueTemp, m_strValueTemp);
        }

        // 통신포트 선택. COM3이 기본
        private void cbPortNo_SelectedIndexChanged(object sender, EventArgs e)
        {
            // SmartComboBox에서 선택한 Item의 Index로 PortNo 속성값을 설정
            if (cmbPortNo.SelectItemIndex == 4)
            {
                smartSerialPort1.PortNo = (SmartX.SmartSerialPort.COMPORTNO)(cmbPortNo.SelectItemIndex + 3);
            }
            else
            {
                smartSerialPort1.PortNo = (SmartX.SmartSerialPort.COMPORTNO)(cmbPortNo.SelectItemIndex + 1);
            }
        }

        // BaudRate 선택. 9600bps가 기본
        private void cbBaudRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            // SmartComboBox에서 선택한 Item의 Index로 Baud_Rate 속성값을 설정
            smartSerialPort1.Baud_Rate = (SmartX.SmartSerialPort.BAUDRATE)Convert.ToInt32(cmbBaudRate.Text);
        }

        // 포트 Open
        private void btnOpen_Click(object sender, EventArgs e)
        {
            if (smartSerialPort1.IsOpen == false)
            {
                // 에러 체크모드는 CRC32
                smartSerialPort1.ErrorCheckMode = SmartX.SmartSerialPort.ERRORCHECK.CRC32;
                // 에러코드 위치는 TAIL
                smartSerialPort1.ErrorCode_Location = SmartX.SmartSerialPort.ERRORCODELOCATION.TAIL;
                // 에러 체크 코드타입은 BINARY
                smartSerialPort1.ErrorCheckCodeType = SmartX.SmartSerialPort.ERRORCHECKCODETYPES.BINARY;
                // 프레임 버퍼 사이즈
                smartSerialPort1.FrameBufferSize = 3000000;
                // 프레임 구분은 STXONLY
                smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.STXONLY_READTIMEOUT;
                byte[] stxCods = new byte[2];
                stxCods[0] = 0x02;
                stxCods[1] = 0x03;
                smartSerialPort1.STXCodes = stxCods;
                // 수신측 타임아웃 시간
                smartSerialPort1.ReadTimeout = 50;
                // 데이터 수신은 이벤트 방식
                smartSerialPort1.ReceiveDetect = SmartX.SmartSerialPort.RECEIVEDETECTTYPE.EVENT_QUEUE;
                // 수신 관련 함수에서 데이터만 출력
                smartSerialPort1.ReceiveFrameDebugMode = false;
                smartSerialPort1.RS485SoftwareDetection = false;

                smartSerialPort1.Open();
                labPortStatus.Text = "Opened";
            }
        }

        // 포트 Close
        private void btnClose_Click(object sender, EventArgs e)
        {
            if (smartSerialPort1.IsOpen == true)
            {
                smartSerialPort1.Close();
                labPortStatus.Text = "Closed";
            }
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            btnClose_Click(sender, e);
            smartForm1.Close();
        }

        // 송신 데이터 변경. int 최대 8자리, float 최대 10자리, string 최대 6자리로 제한
        private void SmartLabel_Click(object sender, EventArgs e)
        {
            smartKeyboard1.Show(123, 269, (SmartX.SmartLabel)sender);
        }

        // Keyboard 입력이 끝났을 때 데이터 생성
        private void smartKeyboard1_OnXKeyClick(object sender, EventArgs e)
        {
            // 일반 데이터 생성(송신용)
            // int형 데이터 (4Byte)
            m_iValue1 = Convert.ToInt32(labInt1.Text);
            // float형 데이터 (4Byte)
            m_fValue1 = Convert.ToSingle(labFloat1.Text);
            // string(Unicode)형 데이터 (8Byte)
            m_strValue1 = labUnicode1.Text;
            MakeNormalData(m_iValue1, m_fValue1, m_strValue1);

            // 구조체 데이터 생성(송신용)
            // int형 데이터 (4Byte)
            m_iValueTemp = Convert.ToInt32(labInt2.Text);
            // float형 데이터 (4Byte)
            m_fValueTemp = Convert.ToSingle(labFloat2.Text);
            // string(Unicode)형 데이터 (최대12Byte)
            m_strValueTemp = labUnicode2.Text;
            MakeStructureData(m_iValueTemp, m_fValueTemp, m_strValueTemp);

            smartKeyboard1.Hide();
            smartForm1.Focus();
        }

        // 데이터 1회 송신
        private void btnSend_Click(object sender, EventArgs e)
        {
            if (smartSerialPort1.IsOpen == true)
            {
                // Send(STX + DATA + CRC32 + ETX) 구조체사용X
                if (smartRadioButton1.Checked == true)
                {
                    smartSerialPort1.WriteFrame(m_bySendData1);
                }
                // Send(STX + DATA + CRC32 + ETX) 구조체사용O
                else
                {
                    smartSerialPort1.WriteFrame(m_bySendData2);
                }
            }
        }

        // 데이터 반복 송신
        private void btnAutoSend_Click(object sender, EventArgs e)
        {
            m_iSendCount = 0;
            // 100 이하(0~10)인경우 수신시 데이터 깨질수있음
            smartTimer1.Interval = 100;
            smartTimer1.Start();
        }

        // 데이터 반복 송신 타이머 Tick 이벤트
        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            if (m_iSendCount < 10)
            {
                btnSend_Click(sender, e);
                m_iSendCount++;
            }
            else
            {
                smartTimer1.Stop();
            }
        }

        // 일반 데이터 생성 메서드
        private void MakeNormalData(int m_iValue1, float m_fValue1, string m_strValue1)
        {
            // 초기화
            labIntArray1.Text = "";
            labFloatArray1.Text = "";
            labUnicodeArray1.Text = "";

            // int형 데이터 (4Byte)
            byte[] bIntData1 = BitConverter.GetBytes(m_iValue1);
            // float형 데이터 (4Byte)
            byte[] bFloatData1 = BitConverter.GetBytes(m_fValue1);
            // string(Unicode)형 데이터 (8Byte)
            byte[] bStrData1 = Encoding.Unicode.GetBytes(m_strValue1);
            byte[] bStrData1Length = new byte[1];
            // Unicode라서 *2해줌
            bStrData1Length[0] = Convert.ToByte(m_strValue1.Length * 2);
            // 구조체 여부 체크
            byte[] bCheckData1 = new byte[1];
            bCheckData1[0] = 0x30;

            // 송신용 일반 데이터 m_bSendData1 생성. 사이즈는 구조체 멤버 변수 사이즈를 전부합한 사이즈
            m_bySendData1 = new byte[bIntData1.Length + bFloatData1.Length + 1 + bStrData1.Length + 1];
            // m_bSendData에 bIntData 복사
            Array.Copy(bIntData1, m_bySendData1, bIntData1.Length);
            // m_bSendData에 bFloatData 연결하여 복사
            Array.Copy(bFloatData1, 0, m_bySendData1, bIntData1.Length, bFloatData1.Length);
            // m_bSendData에 bFloatData 연결하여 복사
            Array.Copy(bStrData1Length, 0, m_bySendData1, bIntData1.Length + bFloatData1.Length, 1);
            // m_bSendData에 bStrData1 연결하여 복사
            Array.Copy(bStrData1, 0, m_bySendData1, (bIntData1.Length + bFloatData1.Length + 1), bStrData1.Length);
            // m_bSendData에 bStrData1 연결하여 복사
            Array.Copy(bCheckData1, 0, m_bySendData1, (bIntData1.Length + bFloatData1.Length + 1 + bStrData1.Length), bCheckData1.Length);

            // int형 멤버변수를 Byte배열로 표현
            for (int i = 0; i < 4; i++)
            {
                labIntArray1.Text += string.Format("{0:X2}", m_bySendData1[i]);
            }

            // float형 멤버변수를 Byte배열로 표현
            for (int i = 4; i < 8; i++)
            {
                labFloatArray1.Text += string.Format("{0:X2}", m_bySendData1[i]);
            }

            // string형 멤버변수를 Byte배열로 표현
            for (int i = 9; i < 9 + bStrData1.Length; i++)
            {
                labUnicodeArray1.Text += string.Format("{0:X2}", m_bySendData1[i]);
            }
        }

        // 구조체 데이터 생성 메서드(멤버변수 int, float, string으로 구성된 구조체)
        private void MakeStructureData(int m_iValueTemp, float m_fValueTemp, string m_strValueTemp)
        {
            // 초기화
            labIntArray2.Text = "";
            labFloatArray2.Text = "";
            labUnicodeArray2.Text = "";

            // 인자로 전달된 데이터를 구조체 멤버변수에 저장
            sSendStruct.m_iValue2 = m_iValueTemp;
            sSendStruct.m_fValue2 = m_fValueTemp;
            sSendStruct.m_strValue2 = m_strValueTemp;

            // m_strValue2의 사이즈. Unicode형태라서 곱하기2
            sSendStruct.m_byLength = (byte)(sSendStruct.m_strValue2.Length * 2);
            // 수신측에서 구조체 데이터인지 체크를 위한 구분 데이터
            byte[] bCheckData2 = new byte[1];
            bCheckData2[0] = 0x32;

            // 송신용 구조체 데이터 m_bSendData2 생성. 사이즈는 구조체 멤버 변수 사이즈를 전부합한 사이즈
            m_bySendData2 = new byte[Marshal.SizeOf(sSendStruct.m_iValue2) + Marshal.SizeOf(sSendStruct.m_fValue2) + 
                Marshal.SizeOf(sSendStruct.m_byLength) + sSendStruct.m_byLength + bCheckData2.Length];
            // Buffer.BlockCopy는 특정 오프셋에서 시작하는 소스 배열에서 특정 오프셋에서 시작하는 대상 배열로 지정된 바이트 수를 복사
            // sSendStruct.m_iValue2를 m_bSendData2에 추가
            System.Buffer.BlockCopy(BitConverter.GetBytes(sSendStruct.m_iValue2), 0, m_bySendData2, 0, sizeof(int));
            // sSendStruct.m_fValue2를 m_bSendData2에 추가
            System.Buffer.BlockCopy(BitConverter.GetBytes(sSendStruct.m_fValue2), 0, m_bySendData2, sizeof(int), sizeof(float));

            // C나 C++의 경우 문자열의 끝을 NULL문자를 확인하여 구분하지만 C#의 경우 문자열의 앞부분에 문자열의 길이를 명시해야함
            // sSendStruct.byLength를 m_bSendData2에 추가
            System.Buffer.BlockCopy(BitConverter.GetBytes(sSendStruct.m_byLength), 0, m_bySendData2, sizeof(int) + sizeof(float), sizeof(byte));
            // sSendStruct.m_strValue2를 m_bSendData2에 추가
            System.Buffer.BlockCopy(Encoding.Unicode.GetBytes(sSendStruct.m_strValue2), 0, m_bySendData2, sizeof(int) + sizeof(float) + sizeof(byte), (sSendStruct.m_strValue2.Length * 2));
            // bCheckData2를 m_bSendData2에 추가
            System.Buffer.BlockCopy(bCheckData2, 0, m_bySendData2, sizeof(int) + sizeof(float) + sizeof(byte) + (sSendStruct.m_strValue2.Length * 2), bCheckData2.Length);

            // 구조체의 int형 멤버변수를 Byte배열로 표현
            for (int i = 0; i < 4; i++)
            {
                labIntArray2.Text += string.Format("{0:X2}", m_bySendData2[i]);
            }

            // 구조체의 float형 멤버변수를 Byte배열로 표현
            for (int i = 4; i < 8; i++)
            {
                labFloatArray2.Text += string.Format("{0:X2}", m_bySendData2[i]);
            }

            // 구조체의 string형 멤버변수를 Byte배열로 표현
            for (int i = 9; i < 9 + (sSendStruct.m_strValue2.Length * 2); i++)
            {
                labUnicodeArray2.Text += string.Format("{0:X2}", m_bySendData2[i]);
            }
        }

        // 수신측. 인코딩(BYTE[] --> 구조체데이터). 리턴 타입이 ByteArray
        static StructureData ByteToStructure(byte[] data)
        {
            StructureData BinaryReader1 = new StructureData();

            // 기본 데이터 형식을 특정 인코딩의 이진 값으로 읽음
            using (BinaryReader reader = new BinaryReader(new MemoryStream(data, false), Encoding.Unicode))
            {
                // 현재 스트림에서 부호있는 4바이트 정수를 읽기
                BinaryReader1.m_iValue2 = reader.ReadInt32();
                // 현재 스트림에서 4바이트 부동 소수점 값을 읽기
                BinaryReader1.m_fValue2 = reader.ReadSingle();
                // 현재 스트림에서 string데이터 읽기
                BinaryReader1.m_strValue2 = reader.ReadString();
            }
            return BinaryReader1;
        }

        // 수신측 처리
        private string[] ReceiveData(byte[] bReadByte)
        {
            // Int형 데이터를 포함하는 Byte[]을 저장
            byte[] bRecvInt1 = new byte[4];
            // float형 데이터 포함하는 Byte[]을 저장
            byte[] bRecvFloat1 = new byte[4];
            // string(Unicode)형 데이터 포함하는 Byte[]을 저장
            byte[] bRecvStr1 = new byte[bReadByte[8]];
            bool chkStruct = false;
            string strRet = "";
            string strRetByte = "";
            List<string> strDataList = new List<string>();

            // 수신 데이터가 구조체 형태인지 일반 데이터인지 구분
            for (int i = 0; i < bReadByte.Length; i++)
            {
                // 일반 데이터
                if (48 == bReadByte[i])
                {
                    chkStruct = false;
                }
                // 구조체 데이터
                else if (50 == bReadByte[i])
                {
                    chkStruct = true;
                }
            }

            try
            {
                // 수신 데이터 파싱하는 부분
                // 일반 데이터(구조체 사용X)
                if (chkStruct == false)
                {
                    // Array.Copy를 사용하여 수신 데이터(bReadByte)를 데이터 형태별로 파싱
                    Array.Copy(bReadByte, bRecvInt1, bRecvInt1.Length);
                    Array.Copy(bReadByte, 4, bRecvFloat1, 0, bRecvFloat1.Length);
                    Array.Copy(bReadByte, 9, bRecvStr1, 0, bRecvStr1.Length);

                    // 수신데이터를 각각의 원본데이터로 변환하여 strRet에 저장
                    strRet += BitConverter.ToInt32(bRecvInt1, 0).ToString() + "  ";
                    strRet += BitConverter.ToSingle(bRecvFloat1, 0).ToString() + "  ";
                    strRet += Encoding.Unicode.GetString(bRecvStr1, 0, bRecvStr1.Length);
                    strDataList.Add(strRet);

                    // Byte배열의 bRecvInt1부분을 string 포맷으로 표현
                    for (int i = 0; i < bRecvInt1.Length; i++)
                    {
                        strRetByte += string.Format("{0:X2}", bRecvInt1[i]);
                        if (i != bRecvInt1.Length - 1)
                        {
                            strRetByte += "-";
                        }
                    }
                    strRetByte += "  ";
                    // Byte배열의 bRecvFloat1부분을 string포맷으로 표현
                    for (int i = 0; i < bRecvFloat1.Length; i++)
                    {
                        strRetByte += string.Format("{0:X2}", bRecvFloat1[i]);
                        if (i != bRecvFloat1.Length - 1)
                        {
                            strRetByte += "-";
                        }
                    }
                    strRetByte += "  ";
                    // Byte배열의 bRecvStr1부분을 string포맷으로 표현
                    for (int i = 0; i < bRecvStr1.Length; i++)
                    {
                        strRetByte += string.Format("{0:X2}", bRecvStr1[i]);
                        if (i != bRecvStr1.Length - 1)
                        {
                            strRetByte += "-";
                        }
                    }
                    strDataList.Add(strRetByte);
                }
                // 구조체 데이터
                else
                {
                    // 수신데이터를 각각의 원본 데이터로 변환하여 strRet에 저장
                    sRecvStruct = ByteToStructure(bReadByte);
                    strRet += sRecvStruct.m_iValue2.ToString() + "  ";
                    strRet += sRecvStruct.m_fValue2.ToString() + "  ";
                    strRet += sRecvStruct.m_strValue2;
                    strDataList.Add(strRet);

                    // Byte배열의 sRecvStruct.m_iValue2 부분을 string포맷으로 표현
                    for (int i = 0; i < 4; i++)
                    {
                        strRetByte += string.Format("{0:X2}", bReadByte[i]);
                        if (i != 3)
                        {
                            strRetByte += "-";
                        }
                    }
                    strRetByte += "  ";
                    // Byte배열의 sRecvStruct.m_fValue2부분을 string포맷으로 표현
                    for (int i = 4; i < 8; i++)
                    {
                        strRetByte += string.Format("{0:X2}", bReadByte[i]);
                        if (i != 7)
                        {
                            strRetByte += "-";
                        }
                    }
                    strRetByte += "  ";
                    // Byte배열의 sRecvStruct.m_strValue2부분을 string포맷으로 표현
                    for (int i = 9; i < 9 + (sRecvStruct.m_strValue2.Length * 2); i++)
                    {
                        strRetByte += string.Format("{0:X2}", bReadByte[i]);
                        if (i != 9 + (sRecvStruct.m_strValue2.Length * 2) - 1)
                        {
                            strRetByte += "-";
                        }
                    }
                    strDataList.Add(strRetByte);
                }
            }
            catch (Exception)
            {
                MessageBox.Show("Exception");
            }
            return strDataList.ToArray();
        }

        // 데이터 수신 이벤트
        private void smartSerialPort1_OnReadQueueEvent()
        {
            // 수신데이터 저장 Byte[]
            byte[] readByte;
            SmartX.SmartSerialPort.FRAMEDATAREADSTATUS eStatus;
            eStatus = smartSerialPort1.ReadQueue(out readByte);

            if (eStatus == SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.VALIDDATA)
            {
                smartListBox1.AddItem(ReceiveData(readByte));
                labRecvCnt.Text = smartListBox1.ItemCount.ToString();
            }
        }

        private void btnListClear_Click(object sender, EventArgs e)
        {
            smartListBox1.ClearAll();
        }
    }
}